# -*- coding: UTF-8 -*-

#   icd_module_cam.py   2021.6.13   kita

#!!  ICADでのカムモデリングを補助する関数群

# 使いかた  使用するスクリプトでimport
#import icd_module_cam as imc   # iCAD_cam関連のモジュール群

# 履歴
# 2021.6.30 「MSG00275 指定された基準座標が対象要素上に存在しません。」対策。
#            SxWindow.setUserViewで視点を変更
# 1024.2.25  def cyl_modeling2(h_list, th_list, r_in, r_out, r_rol, h0, debug = 0):   追加   
#            2D作図データより円筒板カムをモデリングすることを想定

# iCADを操作するための各種camモジュール集
# 呼び出し側で
import clr
clr.AddReference('sxnet')
from sxnet import *
import sys
#import my_module
import os
import binascii         # Binary Fileを16進表記で読み込むmoduleのimport 
import re
import numpy as np
import zen_han      #全角<-->半角変換
import my_tk as mtk
import my_vector as myv
import icd_module as imd


#----------------------------
# icd_module_cam.py関数一覧
#----------------------------



# v0, v1ベクトルと色を指定して直線を引く(2D)
def p2p_line(v0, v1, col0 = '空色'):    
    if col0.find('空') > -1: col = 12  
    elif col0.find('薄橙') > -1: col = 15  
    elif col0.find('灰') > -1: col = 8  
    elif col0.find('緑') > -1: col = 3  
    elif col0.find('薄緑') > -1: col = 10  
    elif col0.find('マゼンダ') > -1: col = 6  
    elif col0.find('橙') > -1: col = 9
    elif col0.find('薄橙') > -1: col = 15  
    elif col0.find('紫') > -1: col = 13  
    elif col0.find('赤') > -1: col = 2 
    elif col0.find('白') > -1: col = 1  
    elif col0.find('黄') > -1: col = 4  
    elif col0.find('黄緑') > -1: col = 11  
    
    SxSys.setPenColor(col)    
    pos0 = SxPos(v0[0],v0[1])
    pos1 = SxPos(v0[0]+v1[0],v0[1]+v1[1])
    seg = SxEntSeg.createLine2D(pos0,pos1)


# シリンダーモデリング　cam, 内径, 外径, CamStroke, 割付時間(角度),dθ, ローラ半径, 基礎高さ、配置角度 
def cyl_modeling(cam, r_in, r_out, s_h, th_h, d_th, r_rol, h0, th0_d = 0.0, debug = 0): # 配置角度 th0_d[deg]
             # Cam内外半径、Stroke,割付角[deg],dθ[deg],ローラ半径,基礎高さ,配置角度[deg]x軸基準,モード  
    s_r_list = []   # ローラー径を加味した実変位リスト
    th_list = []    # rolerとカムの接触点角度リスト
    r_t = r_out     # 展開半径は外径を使用
    pm_sh = s_h/abs(s_h)
    #th0 = np.deg2rad(th0_deg)   # 配置角度[rad]
    
    # アクティブウィンドウを取得する。
    pd = SxWindow.getActive()
    pd.setDim(True)
    pd.setUserView(1)     # User View No.1表示切替
    #SxWindow.setUserView(1)
    
    SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)   # 中線
    SxSys.setPenColor(SxSys.COLOR_GRAY)     # 灰色
    SxSys.setDim(True)     # 3Dに切り替え 画面切り替え

    #    (cam, r_in, r_out, r_t, s_h, th_h, d_th, r_rol, h0) = get_cam()
    r_c = (r_in+r_out)/2    # 板厚中心半径
    th_h_rad = np.deg2rad(th_h)     # 割付角[rad]
    th0_rad = np.deg2rad(th0_d)     # 配置角度[rad]
    l_x = r_out*th_h_rad            # カム展開全長[mm}
    #print(th_h_rad)
    nn = int(th_h/d_th)
    nn = int(np.ceil(th_h/d_th))     # 切り上げ 13.8 -> 14
    if nn > 128:
        print('\n  制限値128を超えています\007\n')
        return

    #z = 0.0
    p00 = [r_in,0.0]; p01 = [r_out,0.0] 
    #p00 = [r_in,0.0,z]; p01 = [r_out,0.0, z] 
    #pos0 = SxPos(r_in, 0.0)
    pos0 = SxPos(p00[0], p00[1])
    rpos_list = []      # 複数閉領域の基準点座標配列
    seg_list = []       # Segment List 閉領域要素の2次元配列。板カムの断面４頂点のリストのリスト
    plan = []
    pos_cf = []           # Cam Follower中心座標　SxPos
    pos_edg = []          # Cam外周端面 SxPos  
    org_list = None #ビューの原点に配置
        
    #debug = 1  # debug mode
    #debug = 0   # 作図実行 mode

    #for i in range(nn): # 0 - 14  
    for i in range(nn+1): # 0 - 14  
        rpos_list.append(pos0)
        t = (i*d_th)/th_h
        if t > 1:
            t = 1
        (s, v, a) = cam.sva(t)
        # Cam Follower中心軌跡
        th_cf = np.deg2rad(th0_d + t*th_h)  #[rad]
        #th_cf = np.deg2rad(th0_d + i*d_th)  #[rad]
        px = r_out * np.cos(th_cf)
        pz = -r_out * np.sin(th_cf)
        py = h0 + s_h*s

        pos_cf.append(SxPos(px,py,pz))

        # 組立平面設置個所の補正(roler中心と設置個所のずれを修正
        th_p = np.arctan2(s_h*v, l_x)      # 圧力角[rad]    外径
        #th_k = (r_rol*np.sin(th_pr)/r_c) * 180/np.pi #補正角度[deg] 弧長さ
        #th_k = np.arcsin(r_rol*np.sin(th_pr)/r_c) * 180/np.pi #補正角度[deg]　x軸投影長さ
        th_k = r_rol*np.sin(th_p)/r_out     # カム中心から見た補正角度[rad]
        #s_r = h0 + s*s_h + pm_hr*r_rol*(1.0-np.cos(th_pr))    # ローラー径を加味した実変位
        s_r = h0 + s*s_h - r_rol*np.cos(th_p)    # ローラー径を加味した実変位
        s_r_list.append(s_r)

        th = th_cf + th_k      # rolerとカムの接触点の角度[rad]
        th_list.append(th)

        # Cam外周端面
        px_edg = r_out*np.cos(th)
        pz_edg = -r_out*np.sin(th)
        py_edg = s_r

        pos_edg.append(SxPos(px_edg,py_edg,pz_edg))

        plan.append(make_pln_yth(th))
        #plan.append(make_pln_yth(th0_rad+th))
        #print(p00, p01) 
        pp = [p00, p01, [r_out,s_r], [r_in,s_r]]
        seg_list.append(sakuzu_pp(*pp))
        #if i % 5 == 0:
        #    input('Next')
        
    rseg = SxEntSeg.createLoft3D(seg_list,rpos_list,org_list,False,False,False,False)   # Loft作成
    #rseg = SxEntSeg.createLoft3D(seg_list,rpos_list,org_list,True,True,True,False)   # Loft作成
    #rseg = SxEntSeg.createLoft3D(seg_list,rpos_list,org_list,True,False,True,False)   # Loft作成
    #rseg = SxEntSeg.createLoft3D(seg_list,rpos_list,org_list,False,True,False,False)   # Loft作成 波板
                                #接線接続断面 開始, 中間, 終了, 実像部品を引き継がない     
    if debug == 0:
        for i in range(round(nn+1)):
            pass
            plan[i].delete()  #組立平面削除
        
        # カムフォロワ中心軌跡ワイヤー
        SxSys.setPenColor(SxSys.COLOR_SKY_BLUE)    # 空色
        SxSys.setPenStyle(SxSys.STYLE_ONE_DOT_CHAIN) # 一点鎖線
        SxSys.setPenWidth(SxSys.WIDTH_THIN)         # 細線

        line_cf = []

        i = 0
        for pos in pos_cf:
            if i == 0:
                i += 1
                pre_pos = pos
                continue
            line_cf.append(SxEntSeg.createLine3D(pre_pos,pos))
            pre_pos = pos
            i += 1
        part = SxEntPart.create(line_cf, "CL_Line", "Cam Follwer CL", 10)

        # カム面エッジワイヤー
        SxSys.setPenColor(SxSys.COLOR_RED)    # 赤色
        SxSys.setPenStyle(SxSys.STYLE_SOLID) # 実線
        SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)  # 中線 
        line_edg = []
        i = 0    
        for pos in pos_edg:
            if i == 0:
                i += 1
                pre_pos = pos
                continue
            line_edg.append(SxEntSeg.createLine3D(pre_pos,pos))
            pre_pos = pos
            i += 1
        
        part = SxEntPart.create(line_edg, "Edge_Line", "Cam 外形エッジ", 10)

    return (s_r_list, th_list)        





#------------------------------------------------------------------------------
#   Loftを使った円筒カム作成関数  y_list, th_listを渡すバージョン。
#   2D空間作図データより円筒板カムをモデリング
#------------------------------------------------------------------------------
def cyl_modeling2(h_list, th_list, r_in, r_out): #, h0): #, debug = 0):
    # 高さリスト,角度リスト[rad],Cam内外半径
    if len(h_list) != len(th_list):
        res= mtk.warning('高さリストとθリストの数が合いません。')
        return None
    if len(h_list) > 128:
        res= mtk.warning('ロフトコマンドの制限値128を超えています。')
        return None
    
    # アクティブウィンドウを取得する。
    pd = SxWindow.getActive()
    #pd.setDim(True) # 3Dに切り替え
    #pd.setUserView(1)     # User View No.1表示切替
    
    SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)   # 中線
    SxSys.setPenColor(SxSys.COLOR_GRAY)     # 灰色
    #SxSys.setDim(True)     # 3Dに切り替え 画面切り替え

    # cyl_modeling2 ------------------------------------------
    z = 0.0 # cyl_modeling2
    p00 = [r_in,0.0]; p01 = [r_out,0.0] 
    pos0 = SxPos(p00[0], p00[1], 0.0)
    rpos_list = []      # 複数閉領域の基準点座標配列
    seg_list = []       # Segment List 閉領域要素の2次元配列。板カムの断面４頂点のリストのリスト
    plan_list = []      # 組立平面リスト。後で削除するのに利用
    org_list = None #ビューの原点に配置

    i= 0
    for th in th_list:  
        rpos_list.append(pos0)  # 閉領域の基準点。常にp0 内側下の点
        # 組立平面作成 
        plan_list.append(make_pln_yth(th))
        pp= [p00, p01, [r_out, h_list[i]], [r_in, h_list[i]]]   # カム断面四角
        seg_list.append(sakuzu_pp(*pp))
        i+= 1

    rseg = SxEntSeg.createLoft3D(seg_list,rpos_list,org_list,False,False,False,False)   # Loft作成 1
                                #接線接続断面 開始, 中間, 終了, 実像部品を引き継がない     
    for plan in plan_list:
        plan.delete()  #組立平面削除

    return rseg   
    # cyl_modeling2 ---------------------------


#------------------------------------------------------------------------------
# 垂直投影を使った分円筒作成  引き数はfloatで渡すべし!
#------------------------------------------------------------------------------
def cyl_modeling3(r_in, r_out, ths, the, h, p0= np.array([0.0,0.0,0.0]), vs= 1):
    # vs Default 平面(x-z)
    # 内半径,外半径,Start角度,有効角度,高さ [mm, rad, np.array], 配置原点,基本ビュータイプ定数
    # vs= 1:平面(x-z)   2:正面(x-y) 3:右側面(z\y)   4:左側面(z-y)   5:背面(x-y) 6:下面(x-z)

    SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)   # 中線
    SxSys.setPenColor(SxSys.COLOR_GRAY)     # 灰色

    ths= np.rad2deg(ths); the= np.rad2deg(the) 
    
    pos0= SxPos(p0[0], p0[1], p0[2])
    v0= np.array([p0[0], p0[1]])
    if vs== 1:  # 平面
        pos1= SxPos(p0[0], p0[1]+h, p0[2])
        xvec = SxVec(1.0, 0.0, 0.0)
        zvec = SxVec(0.0, 1.0, 0.0)
    elif vs== 2:    # 正面
        pos1= SxPos(p0[0], p0[1], p0[2]+h)
        xvec = SxVec(1.0, 0.0, 0.0)
        zvec = SxVec(0.0, 0.0, 1.0)
    elif vs== 3:    # 右側面
        pos1= SxPos(p0[0]+h, p0[1], p0[2])
        xvec = SxVec(0.0, 0.0, -1.0)
        zvec = SxVec(1.0, 0.0, 0.0)
    elif vs== 4:    # 左側面
        pos1= SxPos(p0[0]-h, p0[1], p0[2])
        xvec = SxVec(0.0, 0.0, 1.0)
        zvec = SxVec(-1.0, 0.0, 0.0)
    elif vs== 5:    # 背面
        pos1= SxPos(p0[0], p0[1], p0[2]-h)
        xvec = SxVec(-1.0, 0.0, 0.0)
        zvec = SxVec(0.0, 0.0, -1.0)
    elif vs== 6:    # 下面
        pos1= SxPos(p0[0]+h, p0[1]-h, p0[2])
        xvec = SxVec(1.0, 0.0, 0.0)
        zvec = SxVec(0.0, -1.0, 0.0)

    #plan = SxAsmPlane.create(vs)   # 使えない!!!
    # 組立平面作成
    plan = SxAsmPlane.create(pos0, zvec, xvec)
    
    # 円弧作成
    seg_list= []
    #print(pos0.x, pos0.y,r_in*2,ths,the)
    #return
    seg_list.append(SxEntSeg.createArc2D(pos0,(r_in*2),ths,the))
    seg_list.append(SxEntSeg.createArc2D(pos0,(r_out*2),ths,the))
    v_= np.array([1.0, 0.0])
    t= r_out- r_in
    v1= r_in* myv.rotate2(v_, np.deg2rad(ths))
    v2= t*v1/r_in
    v3= r_in* myv.rotate2(v_, np.deg2rad(ths+the))
    v4= t*v3/r_in
    seg_list.append(imd.p2p_line(v0+v1, v2))
    seg_list.append(imd.p2p_line(v0+v3, v4))

    #垂直投影体セグメントを作成。投影開始位置／終了位置指定。
    #seg_cyl= SxEntSeg.createSwept(seg_list,pos0,pos1,False) # 高さが不定
    #垂直投影体セグメントを作成します。高さ指定。 
    seg_cyl= SxEntSeg.createSwept(seg_list, h, pos0, False);
    # 組立平面を削除します
    SxAsmPlane.delete(plan)
    
    return seg_cyl


# 組立平面作成 y方向軸を含み、x軸方向を 0 とする角度θ[rad]の組立平面を作成
def make_pln_yth(yth, gen=[0.0,0.0,0.0]): # yth=0[rad] -> x軸
    pos = SxPos()
    pos.x = gen[0]
    pos.y = gen[1]
    pos.z = gen[2]
    xvec = SxVec(np.cos(yth), 0.0, -np.sin(yth))
    zvec = SxVec(np.sin(yth), 0.0, np.cos(yth))
    plan = SxAsmPlane.create(pos, zvec, xvec)
       # 組立平面原点座標, 組立平面法線方向ベクトル, 組立平面X軸方向ベクトル
    plan.setActive  # アクティブ組立平面に設定
    return plan

# 
def sakuzu_pp(*pp):  # p = []
    nn = len(pp)
    seg_lst = []
    #print(pp)
    for i in range(nn):
        pos0 = SxPos(pp[i][0],pp[i][1])
        #pos0 = SxPos(pp[i][0],pp[i][1],pp[i][2])
        if i == nn-1:
            pos1 = SxPos(pp[0][0],pp[0][1])
            #pos1 = SxPos(pp[0][0],pp[0][1],pp[0][2])
        else:
            pos1 = SxPos(pp[i+1][0],pp[i+1][1])
            #pos1 = SxPos(pp[i+1][0],pp[i+1][1],pp[i+1][2])
        #print(pos0.x, pos0.y, pos1.x, pos1.y)    
        seg = SxEntSeg.createLine2D(pos0,pos1)
        seg_lst.append(seg)

    return seg_lst


# 展開図作成
def sakuzu_2d(p0, cam, r_in, r_out, r_t, s_h, th_h, d_th, r_rol, h0, *tmp_list):
    s_r_list = tmp_list[0]
    th_list = tmp_list[1]
    nn = int(th_h/d_th)

    pos_rc_list = []    # Roler Center座標リスト
    pos_list = []     # Cam面 2D変位リスト
    #sakuzu2D(cam, r_in, r_out, r_t, s_h, th_h, d_th, r_rol, h0, *s_r_list)
    SxSys.setDim(False)     # 2Dに切り替え 画面切り替え
            
    #p0 = SxPos(0.0,0.0)         # カム原点
    y0 = -30
    p0_idx = SxPos(p0.x, p0.y+y0)   # 目盛り原点
    SxSys.setPenColor(SxSys.COLOR_WHITE)    # 白色
    SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)   # 中線
    
    i = 0

    #for i in range(round(nn+1)):
    #    t = i/nn
    #    x = p0.x + i*(r_t * np.pi * d_th/180)
    #    pos_list.append(SxPos(x, s_r_list[i]+p0.y)) 
    #print(th_list)
    for th in th_list:  # カム面
        x = p0.x + (r_t * th)
        pos_list.append(SxPos(x, s_r_list[i]+p0.y)) 
        i += 1

    seg = SxEntSeg.createSpline2D(pos_list,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0) 
         
    # Roler Center軌跡作図 
    #nn = th_h/d_th
    SxSys.setPenColor(SxSys.COLOR_BLUE_GREEN)    # 青緑
    SxSys.setPenStyle(SxSys.STYLE_ONE_DOT_CHAIN) # 一点鎖線
    SxSys.setPenWidth(SxSys.WIDTH_THIN)            # 細線
    for i in range(round(nn+1)):
        t = i/nn
        (s, v, a) = cam.sva(t)
        y = p0.y + s_h*s + r_rol + h0
        x = p0.x + i*(r_t * np.pi * d_th/180)
        pos_rc_list.append(SxPos(x, y)) 
                
    seg = SxEntSeg.createSpline2D(pos_rc_list,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0) 
 

# 目盛り作成
def make_scale(p0, th_h, r_t, d_th = 1):
    SxSys.setPenColor(SxSys.COLOR_BLUE)
    SxSys.setPenStyle(SxSys.STYLE_SOLID) # 実線
    SxSys.setPenWidth(SxSys.WIDTH_THIN)  # 細線

    for th in range(round(th_h+1)):
        x = p0.x + (r_t * np.pi * th/180) 
        if th % 10 == 0: 
            y = 10.0
            text = [zen_han.Han2Zen(str(th))]     # Index文字　0 10 20...
            pos = SxPos(x, p0.y-5.0)
            seg_text = SxEntSeg.createText(pos, text, True,SxEntSeg.DIM_WORD, SxEntSeg.TEXT_BASE_LB, 0.0)

        elif th % 5 == 0: 
            y = 7.5
        else:
            y = 5.0

        SxEntSeg.createLine2D(SxPos(x,p0.y), SxPos(x, p0.y+y))  # Index縦線

    SxEntSeg.createLine2D(SxPos(p0.x, p0.y), SxPos(x, p0.y))   # Index横線



#----------------------------------------
# icd_module_cam.py テスト用Script
#----------------------------------------

if __name__ == '__main__':

    cam_no = 26; rat = 1.0
    #cam_no = 26; rat = 1.0

    if len(sys.argv) < 2:   #引き数無し !!!引数無 !!!argv
        menu = 0
    else:    # 引数１個 
        menu = int(sys.argv[1])

    if menu == 0:   #cam modeling 
        import universal_cam as uvc
        cam = uvc.Universal_cam(cam_no, 0.0, rat)
        r_in = 280.0; r_out = 292.0; s_h = 40.0; th_h = 20.0; d_th = 2; r_rol = 15.0; h0 = 20.0; th0 = 0.0
        (s_r_list, th_list) = cyl_modeling(cam, r_in, r_out, s_h, th_h, d_th, r_rol, h0, th0, 0)

    elif menu == 1:   #cam modeling debug mode
        import universal_cam as uvc
        cam = uvc.Universal_cam(cam_no, 0.0, rat)
        r_in = 280.0; r_out = 292.0; s_h = 40.0; th_h = 20.0; d_th = 2; r_rol = 15.0; h0 = 20.0; th0 = 0.0
        (s_r_list, th_list) = cyl_modeling(cam, r_in, r_out, s_h, th_h, d_th, r_rol, h0, th0, 1)

    elif menu == 2:     # sakuzu_pp
        p0 = [100.0,0.0]; p1 = [200.0,0.0]; p2 = [200.0,50.0]; p3 = [100.0,50.0]
        pp = (p0, p1, p2, p3)
        seg_list = sakuzu_pp(*pp)

    elif menu== 3:  # cyl_modeling2
        h_list= [10,20,30,40,50,60,70,80,90,100]
        thd_list= [10,20,30,40,50,60,70,80,90,100]
        r_in= 140; r_out= 150 #; h0= 30
        th_list= []
        for thd in thd_list:
            th_list.append(np.deg2rad(thd))
            
        cyl_modeling2(h_list, th_list, r_in, r_out)



"""
cam_no      22: Cycroidal
            25:標準変形台形         26:標準変形正弦        27:標準変形等速度
            45:片停留変形台形(m=1)  48:片停留変形正弦
            51:無停留変形台形       52:無停留変形等速度
            1:早送り(v7!=0)         2:切削送り(等速->停止)
            0:t0～t7を指定する	    3:全ての変数を指定
            1: z   早送り曲線のパラメータ　t3 = 1/(1+z)
            0:*tt   t0-t7 をListで指定する場合に使用する
"""
